var __accessCheck = (obj, member, msg) => {
  if (!member.has(obj))
    throw TypeError("Cannot " + msg);
};
var __privateGet = (obj, member, getter) => {
  __accessCheck(obj, member, "read from private field");
  return getter ? getter.call(obj) : member.get(obj);
};
var __privateAdd = (obj, member, value) => {
  if (member.has(obj))
    throw TypeError("Cannot add the same private member more than once");
  member instanceof WeakSet ? member.add(obj) : member.set(obj, value);
};
var __privateSet = (obj, member, value, setter) => {
  __accessCheck(obj, member, "write to private field");
  setter ? setter.call(obj, value) : member.set(obj, value);
  return value;
};

// src/archetype.ts
var _entities, _components, _excluding, _world;
var _Archetype = class _Archetype {
  constructor({
    entities,
    world,
    components,
    without
  }) {
    __privateAdd(this, _entities, /* @__PURE__ */ new Set());
    __privateAdd(this, _components, void 0);
    __privateAdd(this, _excluding, void 0);
    __privateAdd(this, _world, void 0);
    __privateSet(this, _world, world);
    __privateSet(this, _entities, entities);
    __privateSet(this, _components, components);
    __privateSet(this, _excluding, without);
    world.archetypes.add(this);
  }
  get entities() {
    return __privateGet(this, _entities);
  }
  get components() {
    return __privateGet(this, _components);
  }
  get excluding() {
    return __privateGet(this, _excluding) ?? [];
  }
  matches(entity) {
    const matchesArchetype = __privateGet(this, _components).every((component) => {
      return component in entity;
    });
    const matchesExcluding = __privateGet(this, _excluding)?.some((component) => {
      return component in entity;
    }) ?? false;
    return matchesArchetype && !matchesExcluding;
  }
  addEntity(entity) {
    if (__privateGet(this, _entities).has(entity)) {
      return this;
    }
    if (this.matches(entity)) {
      __privateGet(this, _entities).add(entity);
    }
    return this;
  }
  removeEntity(entity) {
    __privateGet(this, _entities).delete(entity);
    return this;
  }
  clearEntities() {
    __privateGet(this, _entities).clear();
  }
  /**
   * Returns a new archetype based on the current archetype, but excludes the
   * specified components.
   * @param components Components that should **not** be present on the entity
   * @returns
   */
  without(...components) {
    const entities = /* @__PURE__ */ new Set();
    for (const entity of __privateGet(this, _entities)) {
      const matchesWithout = components.every((component) => {
        return component in entity;
      });
      if (matchesWithout) {
        continue;
      }
      entities.add(entity);
    }
    const archetype = new _Archetype({
      entities,
      world: __privateGet(this, _world),
      components: __privateGet(this, _components),
      without: components
    });
    return archetype;
  }
};
_entities = new WeakMap();
_components = new WeakMap();
_excluding = new WeakMap();
_world = new WeakMap();
var Archetype = _Archetype;

export {
  __privateGet,
  __privateAdd,
  Archetype
};
//# sourceMappingURL=chunk-LUBWJBKD.js.map