import {
  Archetype,
  __privateAdd,
  __privateGet
} from "./chunk-LUBWJBKD.js";

// src/world.ts
var _archetypes, _entities;
var World = class {
  constructor() {
    __privateAdd(this, _archetypes, /* @__PURE__ */ new Set());
    __privateAdd(this, _entities, /* @__PURE__ */ new Set());
  }
  get archetypes() {
    return __privateGet(this, _archetypes);
  }
  get entities() {
    return __privateGet(this, _entities);
  }
  clearEntities() {
    __privateGet(this, _entities).clear();
    for (const archetype of __privateGet(this, _archetypes)) {
      archetype.clearEntities();
    }
  }
  archetype(...components) {
    const entities = /* @__PURE__ */ new Set();
    for (const entity of __privateGet(this, _entities)) {
      const matchesArchetype = components.every((component) => {
        return component in entity;
      });
      if (matchesArchetype === true) {
        entities.add(entity);
      }
    }
    const archetype = new Archetype({
      entities,
      world: this,
      components
    });
    return archetype;
  }
  createEntity(entity) {
    const _entity = entity ?? {};
    __privateGet(this, _entities).add(_entity);
    for (const archetype of __privateGet(this, _archetypes)) {
      archetype.addEntity(_entity);
    }
    return _entity;
  }
  deleteEntity(entity) {
    for (const archetype of __privateGet(this, _archetypes)) {
      archetype.removeEntity(entity);
    }
    return __privateGet(this, _entities).delete(entity);
  }
  addEntityComponents(entity, component, value) {
    const existingEntity = __privateGet(this, _entities).has(entity);
    if (existingEntity === false) {
      throw new Error(`Entity ${entity} does not exist`);
    }
    entity[component] = value;
    for (const archetype of __privateGet(this, _archetypes)) {
      if (archetype.matches(entity)) {
        archetype.addEntity(entity);
      } else {
        archetype.removeEntity(entity);
      }
    }
    return entity;
  }
  removeEntityComponents(entity, ...components) {
    if (__privateGet(this, _entities).has(entity)) {
      for (const component of components) {
        delete entity[component];
      }
      for (const archetype of __privateGet(this, _archetypes)) {
        if (archetype.matches(entity)) {
          archetype.addEntity(entity);
        } else {
          archetype.removeEntity(entity);
        }
      }
    }
  }
};
_archetypes = new WeakMap();
_entities = new WeakMap();

export {
  World
};
//# sourceMappingURL=chunk-N4FP62KU.js.map