"use strict";
var __defProp = Object.defineProperty;
var __getOwnPropDesc = Object.getOwnPropertyDescriptor;
var __getOwnPropNames = Object.getOwnPropertyNames;
var __hasOwnProp = Object.prototype.hasOwnProperty;
var __export = (target, all) => {
  for (var name in all)
    __defProp(target, name, { get: all[name], enumerable: true });
};
var __copyProps = (to, from, except, desc) => {
  if (from && typeof from === "object" || typeof from === "function") {
    for (let key of __getOwnPropNames(from))
      if (!__hasOwnProp.call(to, key) && key !== except)
        __defProp(to, key, { get: () => from[key], enumerable: !(desc = __getOwnPropDesc(from, key)) || desc.enumerable });
  }
  return to;
};
var __toCommonJS = (mod) => __copyProps(__defProp({}, "__esModule", { value: true }), mod);
var __accessCheck = (obj, member, msg) => {
  if (!member.has(obj))
    throw TypeError("Cannot " + msg);
};
var __privateGet = (obj, member, getter) => {
  __accessCheck(obj, member, "read from private field");
  return getter ? getter.call(obj) : member.get(obj);
};
var __privateAdd = (obj, member, value) => {
  if (member.has(obj))
    throw TypeError("Cannot add the same private member more than once");
  member instanceof WeakSet ? member.add(obj) : member.set(obj, value);
};
var __privateSet = (obj, member, value, setter) => {
  __accessCheck(obj, member, "write to private field");
  setter ? setter.call(obj, value) : member.set(obj, value);
  return value;
};

// src/index.ts
var src_exports = {};
__export(src_exports, {
  World: () => World
});
module.exports = __toCommonJS(src_exports);

// src/archetype.ts
var _entities, _components, _excluding, _world;
var _Archetype = class _Archetype {
  constructor({
    entities,
    world,
    components,
    without
  }) {
    __privateAdd(this, _entities, /* @__PURE__ */ new Set());
    __privateAdd(this, _components, void 0);
    __privateAdd(this, _excluding, void 0);
    __privateAdd(this, _world, void 0);
    __privateSet(this, _world, world);
    __privateSet(this, _entities, entities);
    __privateSet(this, _components, components);
    __privateSet(this, _excluding, without);
    world.archetypes.add(this);
  }
  get entities() {
    return __privateGet(this, _entities);
  }
  get components() {
    return __privateGet(this, _components);
  }
  get excluding() {
    return __privateGet(this, _excluding) ?? [];
  }
  matches(entity) {
    const matchesArchetype = __privateGet(this, _components).every((component) => {
      return component in entity;
    });
    const matchesExcluding = __privateGet(this, _excluding)?.some((component) => {
      return component in entity;
    }) ?? false;
    return matchesArchetype && !matchesExcluding;
  }
  addEntity(entity) {
    if (__privateGet(this, _entities).has(entity)) {
      return this;
    }
    if (this.matches(entity)) {
      __privateGet(this, _entities).add(entity);
    }
    return this;
  }
  removeEntity(entity) {
    __privateGet(this, _entities).delete(entity);
    return this;
  }
  clearEntities() {
    __privateGet(this, _entities).clear();
  }
  /**
   * Returns a new archetype based on the current archetype, but excludes the
   * specified components.
   * @param components Components that should **not** be present on the entity
   * @returns
   */
  without(...components) {
    const entities = /* @__PURE__ */ new Set();
    for (const entity of __privateGet(this, _entities)) {
      const matchesWithout = components.every((component) => {
        return component in entity;
      });
      if (matchesWithout) {
        continue;
      }
      entities.add(entity);
    }
    const archetype = new _Archetype({
      entities,
      world: __privateGet(this, _world),
      components: __privateGet(this, _components),
      without: components
    });
    return archetype;
  }
};
_entities = new WeakMap();
_components = new WeakMap();
_excluding = new WeakMap();
_world = new WeakMap();
var Archetype = _Archetype;

// src/world.ts
var _archetypes, _entities2;
var World = class {
  constructor() {
    __privateAdd(this, _archetypes, /* @__PURE__ */ new Set());
    __privateAdd(this, _entities2, /* @__PURE__ */ new Set());
  }
  get archetypes() {
    return __privateGet(this, _archetypes);
  }
  get entities() {
    return __privateGet(this, _entities2);
  }
  clearEntities() {
    __privateGet(this, _entities2).clear();
    for (const archetype of __privateGet(this, _archetypes)) {
      archetype.clearEntities();
    }
  }
  archetype(...components) {
    const entities = /* @__PURE__ */ new Set();
    for (const entity of __privateGet(this, _entities2)) {
      const matchesArchetype = components.every((component) => {
        return component in entity;
      });
      if (matchesArchetype === true) {
        entities.add(entity);
      }
    }
    const archetype = new Archetype({
      entities,
      world: this,
      components
    });
    return archetype;
  }
  createEntity(entity) {
    const _entity = entity ?? {};
    __privateGet(this, _entities2).add(_entity);
    for (const archetype of __privateGet(this, _archetypes)) {
      archetype.addEntity(_entity);
    }
    return _entity;
  }
  deleteEntity(entity) {
    for (const archetype of __privateGet(this, _archetypes)) {
      archetype.removeEntity(entity);
    }
    return __privateGet(this, _entities2).delete(entity);
  }
  addEntityComponents(entity, component, value) {
    const existingEntity = __privateGet(this, _entities2).has(entity);
    if (existingEntity === false) {
      throw new Error(`Entity ${entity} does not exist`);
    }
    entity[component] = value;
    for (const archetype of __privateGet(this, _archetypes)) {
      if (archetype.matches(entity)) {
        archetype.addEntity(entity);
      } else {
        archetype.removeEntity(entity);
      }
    }
    return entity;
  }
  removeEntityComponents(entity, ...components) {
    if (__privateGet(this, _entities2).has(entity)) {
      for (const component of components) {
        delete entity[component];
      }
      for (const archetype of __privateGet(this, _archetypes)) {
        if (archetype.matches(entity)) {
          archetype.addEntity(entity);
        } else {
          archetype.removeEntity(entity);
        }
      }
    }
  }
};
_archetypes = new WeakMap();
_entities2 = new WeakMap();
// Annotate the CommonJS export names for ESM import in node:
0 && (module.exports = {
  World
});
//# sourceMappingURL=index.cjs.map